/*
 * System Interface Library for games
 * Copyright (c) 2007-2017 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/random.c: Random number generation routines.
 */

#include "src/base.h"
#include "src/random.h"
#include "src/sysdep.h"

/*************************************************************************/
/****************************** Local data *******************************/
/*************************************************************************/

/* Global seed (state) value for random() and urandom().  Initialized to 1
 * to match the C99 standard. */
static uint64_t random_state = 1;

/*************************************************************************/
/************************** Interface routines ***************************/
/*************************************************************************/

/*
 * These functions are declared inline in random.h.  These "extern inline"
 * declarations tell the compiler to emit a copy of the function in case a
 * non-inlined version is needed, such as if a pointer to the function is
 * taken or if a non-C99 source file includes <SIL/random.h>.
 */

extern inline long random_r(uint64_t *state);
extern inline unsigned long urandom_r(uint64_t *state);
extern inline int64_t random64_r(uint64_t *state);
extern inline uint64_t urandom64_r(uint64_t *state);
extern inline double frandom_r(uint64_t *state);
extern inline float frandomf_r(uint64_t *state);

/*-----------------------------------------------------------------------*/

long random(void)
{
    return random_r(&random_state);
}

/*-----------------------------------------------------------------------*/

unsigned long urandom(void)
{
    return urandom_r(&random_state);
}

/*-----------------------------------------------------------------------*/

int64_t random64(void)
{
    return random64_r(&random_state);
}

/*-----------------------------------------------------------------------*/

uint64_t urandom64(void)
{
    return urandom64_r(&random_state);
}

/*-----------------------------------------------------------------------*/

double frandom(void)
{
    return frandom_r(&random_state);
}

/*-----------------------------------------------------------------------*/

float frandomf(void)
{
    return frandomf_r(&random_state);
}

/*-----------------------------------------------------------------------*/

void srandom(unsigned int seed)
{
    random_state = (uint64_t)seed;
}

/*-----------------------------------------------------------------------*/

void srandom64(uint64_t seed)
{
    random_state = seed;
}

/*-----------------------------------------------------------------------*/

void srandom_env(void)
{
    /* Add rather than simply assigning, so that (for example) if
     * sys_random_seed() returns the same value for calls within the same
     * second, we still get a different random sequence. */
    random_state += sys_random_seed();
}

/*************************************************************************/
/*************************************************************************/

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
